
#include <GL/gl.h>

#ifndef M_PI
#define M_PI 3.14159265
#endif

/* max amount of textures */
#define MAX_TEXTURES 64

#define SCREEN_X 1600
#define SCREEN_Y 1200

#define OFF 0
#define ON 1

#define FAILED 0
#define SUCCEEDED 1

#define NO 0
#define YES 1

#define KEY_STATUS_LEFT  1
#define KEY_STATUS_RIGHT 2
#define KEY_STATUS_UP    4
#define KEY_STATUS_DOWN  8

/*******************************************/
/* lsload.c                                */
/*******************************************/

struct object_object {
  float x, y, z;
  int display_list;
  struct object_mesh *meshes;
  struct object_object *next;
};

struct object_mesh {
  char name[64];
  int vertices;
  int normals;
  int triangles;
  int collision_status;
  int *triangle_list;
  int *textures;
  int *triangle_normal_list;
/* landscape creator variables */
  int *triangle_cube;
  double *triangle_center_xyz;
/* --------------------------- */
  double *vertice_list;
  double bounding_sphere_r;
  float  *normal_list;
  float  *uv_list;
  float  *triangle_radius_list;
  float  *triangle_collision_normal_list;
  struct object_mesh *next;
};

/*******************************************/
/* txt.c                                   */
/*******************************************/

#define TXT_STATUS_NORMAL       0
#define TXT_STATUS_ALPHA        1
#define TXT_STATUS_NO_COLLISION 2

struct txt_object {
  int id;
  int x, y;
  int dirty;
  int status;
  int size;      /* the size of the texture data */
  int type;      /* opengl type */
  char *txt;
  char name[64];
  float alpha;
  GLuint ogl_name;
  double friction;
  double speed;
  unsigned char r, g, b;
};

struct txt_extra_info {
  int collision;
  char name[256];
  float alpha;
  unsigned char r, g, b;
  double friction, speed;
  struct txt_extra_info *next;
};

/*******************************************/
/* particles.c                             */
/*******************************************/

#define PARTICLES_MAX 512

struct particle {
  int age;
  float d;
  float sc;
  float x, y, z;
  float vx, vy, vz;
  float ax, ay, az;
  float cr, cg, cb;
};

#define PARTICLE_SYSTEM_POINT_FLAME 0
#define PARTICLE_SYSTEM_POINT_BLOOD 1
#define PARTICLE_SYSTEM_FACE_MIST 2

struct particle_system {
  int type;
  int id;
  int amount;
  float x, y, z;
  float dx, dy, dz;
  float *tri;
  struct particle *particles;
  struct particle_system *prev, *next;
};
