
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "defines.h"
#include "lsload.h"
#include "txt.h"

/*
#define LS_DEBUG 1
*/

extern struct object_mesh *meshes_first, *meshes_last;
extern int txt_id_conversion, lsload_errors;


int object_load_landscape(char *name) {

  struct object_mesh *mesh;
  struct txt_object *txt;
  double *d;
  float *l;
  FILE *f;
  char t[256], mn[256];
  int m, i, *a, *b, *c, vertices, normals, triangles, g, h, txt_list[256];


  f = fopen(name, "rb");
  if (f == NULL) {
    if (lsload_errors == ON)
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: Could not open file \"%s\".\n", name);
    return FAILED;
  }

#ifdef LS_DEBUG
  printf("FILE \"%s\"\n", name);
#endif

  meshes_first = NULL;
  meshes_last = NULL;

  while (1) {
    fscanf(f, "%s", t);
    if (strcmp("Meshes:", t) == 0) {
      fscanf(f, "%d", &m);
      break;
    }
  }

  if (m == 0) {
    fclose(f);
    return SUCCEEDED;
  }

  /* get meshes */
  while (m > 0) {
    mesh = malloc(sizeof(struct object_mesh));
    if (mesh == NULL) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: Out of memory.\n");
      return FAILED;
    }

    /* discard "// Mesh data" */
    fscanf(f, "%*s %*s %*s");

    /* mesh name */
    fscanf(f, "%s", t);
    if (t[0] == '"') {
      strcpy(mesh->name, &t[1]);
      mesh->name[strlen(mesh->name) - 1] = 0;
    }
    else
      strcpy(mesh->name, t);

#ifdef LS_DEBUG
    printf("mesh name: %s\n", mesh->name);
#endif

    /* vertices */
    fscanf(f, "%s", t);
    if (strcmp("Vertices:", t) != 0) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: No \"Vertices:\" found!\n");
      fclose(f);
      return FAILED;
    }
    fscanf(f, "%d", &vertices);

#ifdef LS_DEBUG
    printf("vertices: %d\n", vertices);
#endif

    /* normals */
    fscanf(f, "%s", t);
    if (strcmp("Normals:", t) != 0) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: No \"Normals:\" found!\n");
      fclose(f);
      return FAILED;
    }
    fscanf(f, "%d", &normals);

#ifdef LS_DEBUG
    printf("normals: %d\n", normals);
#endif

    /* triangles */
    fscanf(f, "%s", t);
    if (strcmp("Triangles:", t) != 0) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: No \"Triangles:\" found!\n");
      fclose(f);
      return FAILED;
    }
    fscanf(f, "%d", &triangles);

#ifdef LS_DEBUG
    printf("triangles: %d\n", triangles);
#endif

    /* textures, triangle normal list, triangle list */
    i = sizeof(int) * (triangles + triangles*3 + triangles*3);

    mesh->textures = malloc(i);
    if (mesh->textures == NULL) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: Out of memory.\n");
      free(mesh);
      return FAILED;
    }
    mesh->triangle_normal_list = mesh->textures + triangles;
    mesh->triangle_list = mesh->triangle_normal_list + triangles*3;

    mesh->uv_list = malloc(sizeof(float) * triangles * 2 * 3);
    if (mesh->uv_list == NULL) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: Out of memory.\n");
      free(mesh->textures);
      free(mesh);
      return FAILED;
    }
    mesh->vertice_list = malloc(sizeof(double) * vertices * 3);
    if (mesh->vertice_list == NULL) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: Out of memory.\n");
      free(mesh->uv_list);
      free(mesh->textures);
      free(mesh);
      return FAILED;
    }

    if (normals > 0) {
      mesh->normal_list = malloc(sizeof(float) * normals * 3);
      if (mesh->normal_list == NULL) {
	fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: Out of memory.\n");
	free(mesh->uv_list);
	free(mesh->textures);
	free(mesh->vertice_list);
	free(mesh);
	return FAILED;
      }
    }

    mesh->next = NULL;
    mesh->vertices = vertices;
    mesh->normals = normals;
    mesh->triangles = triangles;

    /* discard "// Vertice data" */
    fscanf(f, "%*s %s %*s", t);
    if (strcmp("Vertice", t) != 0) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: File \"%s\", no \"Vertice\" found! Got \"%s\" instead.\n", name, t);
      fclose(f);
      return FAILED;
    }

    d = mesh->vertice_list;
    for (i = 0; i < (vertices>>1); i++) {
      fscanf(f, "%lf %lf %lf %lf %lf %lf", d, d+1, d+2, d+3, d+4, d+5);
      d += 6;
    }
    if ((vertices & 1) == 1)
      fscanf(f, "%lf %lf %lf", d, d+1, d+2);

    if (normals > 0) {
      /* discard "// Normal data" */
      fscanf(f, "%*s %*s %*s");

      l = mesh->normal_list;
      for (i = 0; i < (normals>>1); i++) {
	fscanf(f, "%f %f %f %f %f %f", l, l+1, l+2, l+3, l+4, l+5);
	l += 6;
      }
      if ((normals & 1) == 1)
	fscanf(f, "%f %f %f", l, l+1, l+2);
    }

    /* discard "// Triangle data" */
    fscanf(f, "%*s %s %*s", t);
    if (strcmp("Triangle", t) != 0) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: File \"%s\", no \"Triangle\" found! Got \"%s\" instead.\n", name, t);
      fclose(f);
      return FAILED;
    }

    /* has the mesh normals? */
    if (normals > 0) {
      a = mesh->triangle_list;
      b = mesh->textures;
      c = mesh->triangle_normal_list;
      l = mesh->uv_list;
      for (i = 0; i < (triangles>>1); i++) {
	fscanf(f, "%d %d %d %d %d %d %d %f %f %f %f %f %f", b, a, a+1, a+2, c, c+1, c+2, l, l+1, l+2, l+3, l+4, l+5);
	fscanf(f, "%d %d %d %d %d %d %d %f %f %f %f %f %f", b+1, a+3, a+4, a+5, c+3, c+4, c+5, l+6, l+7, l+8, l+9, l+10, l+11);
	b += 2;
	a += 6;
	c += 6;
	l += 12;
      }
      if ((triangles & 1) == 1)
	fscanf(f, "%d %d %d %d %d %d %d %f %f %f %f %f %f", b, a, a+1, a+2, c, c+1, c+2, l, l+1, l+2, l+3, l+4, l+5);
    }
    /* no normals ;( */
    else {
      a = mesh->triangle_list;
      b = mesh->textures;
      l = mesh->uv_list;
      for (i = 0; i < triangles; i++) {
	fscanf(f, "%d %d %d %d %f %f %f %f %f %f", b, a, a+1, a+2, l, l+1, l+2, l+3, l+4, l+5);
	b++;
	a += 3;
	l += 6;
      }
    }

    /* discard "// Texture data" */
    fscanf(f, "%*s %s %*s", t);
    if (strcmp("Texture", t) != 0) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: File \"%s\", no \"Texture\" found! Got \"%s\" instead.\n", name, t);
      fclose(f);
      return FAILED;
    }

    /* textures */
    fscanf(f, "%s", t);
    if (strcmp("Textures:", t) != 0) {
      fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: File \"%s\", no \"Texturess:\" found! Got \"%s\" instead.\n", name, t);
      fclose(f);
      return FAILED;
    }
    fscanf(f, "%d", &g);

    /* load textures */
    h = 0;
    while (g > 0) {
      /* texture name */
      fscanf(f, "%s", t);
      if (t[0] == '"') {
	strcpy(mn, &t[1]);
	mn[strlen(mn) - 1] = 0;
      }
      else
	strcpy(mn, t);

      txt_load(mn, &i);
      txt_list[h] = i;
      h++;
      if (txt_get(i, &txt) != SUCCEEDED) {
	fprintf(stderr, "OBJECT_LOAD_LANDSCAPE: Unknown texture \"%s\".\n", t);
	fclose(f);
	return FAILED;
      }

      g--;
    }

    /* convert local texture ids to local texture ids */
    if (txt_id_conversion == ON) {
      b = mesh->textures;
      for (i = 0; i < triangles; i++) {
	*b = txt_list[*b];
	b++;
      }
    }

    m--;

    /* add this mesh to the mesh list */
    if (meshes_first == NULL) {
      meshes_first = mesh;
      meshes_last = mesh;
    }
    else {
      meshes_last->next = mesh;
      meshes_last = mesh;
    }
  }

  fclose(f);

  return SUCCEEDED;
}
